; @(#) glide.el --- Configuration of Emacs to use for ACT's Glide environment

;; Copyright (C) 2000 Ada Core Technologies

;; Author: Emmanuel Briot <briot@gnat.com>
;; Maintainer: Emmanuel Briot <briot@gnat.com>
;; Ada Core Technologies's version:   $Revision: 1.47.2.6 $
;; Keywords: languages ada glide

;; This file is not part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:


;;;???  jka-compre
;;;??? emacs.places  (options)
;;;??? desktop-save  (options)

;;*****************************************************
;;** Set some specific variables when Emacs is not installed in the
;;** directory it was compiled for
;;*****************************************************

(set 'doc-directory  data-directory)


;;*****************************************************
;;** General configuration of Emacs for Glide
;;*****************************************************

(line-number-mode 1)      ;;; Show the line number in the mode line
(column-number-mode 1)    ;;; Show the column number in the mode line

(if (and (boundp 'running-xemacs) (eval 'running-xemacs)) ;;; Show matching parenthesis
    (progn
      (load "paren")
      (funcall (symbol-function 'paren-set-mode) 'paren)
      (set 'zmacs-regions t))
  (show-paren-mode 1)
  (transient-mark-mode t))

(setq scroll-step 1)      ;;; Smooth scrolling (one line at a time)
(setq scroll-conservatively 10) ;;; Scroll at most that many lines
(setq inhibit-startup-message t) ;;; No startup message with "Emacs" in it
(setq next-line-add-newlines nil) ;; don't add newlines with 'down' key

(if (functionp 'blink-cursor-mode)
    (blink-cursor-mode -1))  ;; no blinking cursor

;;  Set the title of the frames: when a project file is defined, put its
;;  name in the frame title.
;;  If the buffer does not have a file name, display its name instead
(setq frame-title-format
      '(""
	(ada-prj-default-project-file
	 ("  (Prj: " ada-prj-default-project-file ")"))
	(buffer-file-name "%f" "%b")
	))
(setq icon-title-format "%b")

(set 'c-frame nil)  ;;; fix a bug in speedbar on Emacs 20.3
(unless (and (boundp 'running-xemacs) (eval 'running-xemacs))
  (define-key (lookup-key global-map [menu-bar files]) [exit-emacs]
    '("Exit" . save-buffers-kill-emacs)))  ;; Rename the entry to hide "Emacs"

;; Sort the entries in speedbar
;; Note that for speedbar to work, we need to have
;;    speedbar-use-imenu-flag to non-nil value
(require 'speedbar)
;;  (set 'speedbar-tag-hierarchy-method '(prefix-group trim-words sort))

;;  Some special files to open in Ada mode
(add-to-list 'auto-mode-alist '("\\.adc\\'" . ada-mode))

;; Windows-specific settings for directory separators.
;; This is a complex platforms, since various uncompatible shells are used
;; by different people. Some of them will correctly understand forward
;; slashes for directory separators, some wont. We thus try the simplest
;; algorithm to see which separator to use.

(if (and (string-match "windows" (symbol-name system-type))
	 (or (string-match "cmdproxy"     shell-file-name)
	     (string-match "cmd.exe"      shell-file-name)
	     (string-match "command.com"  shell-file-name)))
    (setq directory-sep-char ?\\))

(defconst glide-gnatdist-ext "cfg"
  "Extension for Gnatdist configuration files.")

(defconst glide-project-ext "gpr"
  "Extension for project files.")

(defconst glide-version-file
  (concat
   (file-name-directory (locate-library "ada-mode"))
   "glide_version.txt")
  "Name of the file that contains the version number.")

;;*******************************************************
;;** Support for the .gpr project files
;;*******************************************************

(require 'glide-external "glide-external" t)

(defun glide-load-gpr-project-file (filename)
  "Function run from `ada-load-project-hook', to provide support for
loading the new .gpr files in Emacs."
  (let (prj-buffer)
    (condition-case nil
	(if (string= (file-name-extension filename) "gpr")
	    (glide-external-synchronous
	     "convert_prj" (list filename)
	     (lambda () (setq prj-buffer (current-buffer)))
	     t))
      (error (setq prj-buffer nil)))
    prj-buffer))

(if (locate-library "glide-external")
    (add-hook 'ada-load-project-hook 'glide-load-gpr-project-file)
  )


;;*******************************************************
;;** Support functions
;;*******************************************************

(defun glide-add-menu (map name)
  "Add some Glide specific items in the menu NAME.
NAME should be the same name that was given to `ada-create-menu'."
  (if (and (boundp 'running-xemacs) (eval 'running-xemacs))
      ;; For XEmacs
      (funcall (symbol-function 'add-menu-button)
	       (list (symbol-name name)
		     ["Speedbar" speedbar-frame-mode
		      :style toggle
		      :selected (and (boundp 'speedbar-frame)
				     (frame-live-p speedbar-frame)
				     (frame-visible-p speedbar-frame))]
		     "Index"))
    ;; for Emacs
    (define-key-after (lookup-key map (vector 'menu-bar name)) [Speedbar]
      '("Speedbar" . speedbar-get-focus) 'Index)))
  

(defun glide-ada-mode ()
  "Load the ada-mode, with some customization"

  (set 'ada-which-compiler 'gnat)  ;; If using GLIDE, we are using GNAT :-)
  (require 'ada-mode)
  (glide-add-menu ada-mode-map 'Ada)

  (define-key (lookup-key ada-mode-map [menu-bar Ada Help])
    [glide] '("Glide" . (lambda() (interactive) (info "(glide)"))))
  (define-key (lookup-key ada-mode-map [menu-bar Ada Help])
    [ddd] '("gvd" . (lambda() (interactive) (info "(gvd)"))))
  (define-key (lookup-key ada-mode-map [menu-bar Ada Help]) [Ada\ Mode]
    'undefined)
  
  ;; ---  Imenu support  ---
  (require 'imenu)
  ;; (setq imenu-auto-rescan t)   ;;; set to t if we should always rescan
  ;; (setq imenu-use-markers nil) ;;; if we automatically rescan, use positions
  (set 'imenu-scanning-message nil) ;;; no message should be displayed
  (add-hook 'ada-mode-hook (lambda() (imenu-add-to-menubar "Entities")))

  ;; ---  Gnatdist files  ---
  (add-to-list 'auto-mode-alist (cons (concat (regexp-quote glide-gnatdist-ext) "\\'")
				      'ada-mode))
  (add-hook 'ada-mode-hook 'glide-gnatdist-support)

  ;; ---  New project files ---
  (add-to-list 'auto-mode-alist (cons (concat (regexp-quote glide-project-ext) "\\'")
				      'ada-mode))
  (add-hook 'ada-mode-hook 'glide-project-support)

  ;;  Load the support package for AUnit
  (if (locate-library "ada-aunit")
      (require 'ada-aunit))
  
  ;;  This needs to be done here, after the ada-mode has been loaded.
  (if (locate-library "glide-debug")
      (require 'glide-debug))
  
  ;;  Create the Glide menu. The Ada menu has already been created.
  (unless (lookup-key ada-mode-map [menu-bar Glide])
    (define-key global-map [menu-bar Glide]
      (cons "Glide" (copy-keymap (lookup-key ada-mode-map [menu-bar Ada]))))
    (set 'menu-bar-final-items (append '(Glide) menu-bar-final-items))

    (define-key-after (lookup-key global-map [menu-bar edit])
      [goto-line] '("Goto Line..." . goto-line) 'clear)
    
    ;; No Glide menu in Ada-mode, there's already the Ada menu
    (define-key ada-mode-map [menu-bar Glide] 'undefined)
    )
  )


(defun glide-font-lock ()
  "Activate the color highlighting mode"
  (set 'font-lock-maximum-size nil)
  (if (and (boundp 'running-xemacs) (eval 'running-xemacs))
      (set 'font-lock-auto-fontify t)
      ;;(add-hook 'ada-mode-hook '(lambda() (font-lock-mode t)))
    (global-font-lock-mode t)))

(defun glide-gnatdist-support ()
  "Install the support for gnatdist configuration files in the current buffer.
Nothing is done if the current file's extension is not gnatdist's extension."

  (if (and (buffer-file-name)
	   (string= (file-name-extension (buffer-file-name))
		    glide-gnatdist-ext))
      (progn
	(set (make-local-variable 'ada-package-start-regexp)
	     "^[ \t]*\\(package\\|configuration\\)")

	;; Set up the keywords. Note that in (ada-mode), the list of keywords
	;; is modified after the hook has been run, this is why we modify
	;; directly ada-95-keywords instead of ada-keywords
	(set 'ada-language-version 'ada95)
	(set (make-local-variable 'ada-95-keywords)
	     (concat "\\(" ada-95-keywords "\\|\\<configuration\\>\\)"))
	(make-local-variable 'ada-keywords)

	;;  So that "end <name>" is associated with "configuration <name>"
	(set (make-local-variable 'ada-matching-start-re)
	     (concat ada-matching-start-re "\\|\\<configuration\\>"))
	(set (make-local-variable 'ada-end-stmt-re)
	     (concat ada-end-stmt-re "\\|[ \t]*\\<configuration\\>"))
	
	;;  Correct indentation of the variable declarations,...
	(set (make-local-variable 'ada-subprog-start-re)
	     (concat ada-subprog-start-re "\\|\\<configuration\\>"))
	
	(make-local-variable 'ada-font-lock-keywords)
	(add-to-list 'ada-font-lock-keywords"\\<configuration\\>")
	)
    ))

(defun glide-project-support ()
  "Install the support for project files in the current buffer.
Nothing is done if the current file's extension is not glide-project-ext."

  (if (and (buffer-file-name)
	   (string= (file-name-extension (buffer-file-name))
		    glide-project-ext))
      (progn
	(set (make-local-variable 'ada-package-start-regexp)
	     "^[ \t]*\\(package\\|project\\)")

	;; Set up the keywords. Note that in (ada-mode), the list of keywords
	;; is modified after the hook has been run, this is why we modify
	;; directly ada-95-keywords instead of ada-keywords
	(set 'ada-language-version 'ada95)
	(set (make-local-variable 'ada-95-keywords)
	     (concat "\\(" ada-95-keywords
		     "\\|\\<\\(project\\|modifying\\|external\\)\\>\\)"))
	(make-local-variable 'ada-keywords)

	;;  So that "end <name>" is associated with "configuration <name>"
	(set (make-local-variable 'ada-matching-start-re)
	     (concat ada-matching-start-re "\\|\\<project\\>"))
	(set (make-local-variable 'ada-end-stmt-re)
	     (concat ada-end-stmt-re "\\|[ \t]*\\<project\\>"))
	
	;;  Correct indentation of the variable declarations,...
	(set (make-local-variable 'ada-subprog-start-re)
	     (concat ada-subprog-start-re "\\|\\<project\\>"))
	
	(make-local-variable 'ada-font-lock-keywords)
	(add-to-list 'ada-font-lock-keywords "\\<project\\>")
	(add-to-list 'ada-font-lock-keywords "\\<modifying\\>")
	(add-to-list 'ada-font-lock-keywords "\\<external\\>")
	)
    ))

(defun glide-add-extensions (extensions)
  "Add a new possible set of extensions for Ada files.
EXTENSIONS is a string made of space-separated items \"spec,body\".
The string must end with a space"
  
  (let ((i 0) (beg 0) word spec body comma
	(len (length extensions)))
    (while (< i len)
      (if (= (aref extensions i) ? )
	  (progn
	    (setq word (substring extensions beg i)
		  beg  (1+ i))
	    (while (and (< beg len) (= (aref extensions beg) ? ))
	      (setq beg (1+ beg)))
	    (setq i (1+ beg))

	    ;; Now we have a word of the form "spec,body"
	    (setq comma 0)
	    (while (< comma (length word))
	      (if (= (aref word comma) ?,)
		  (progn
		    (setq spec (substring word 0 comma)
			  body (substring word (1+ comma)))
		    (ada-add-extensions spec body)
		    (setq comma (length word))
		    ))
	      (setq comma (1+ comma)))
	    )
	(setq i (1+ i))))
  ))


(defun glide-explorer ()
  "Activate Speedbar.
Preserves the current frame."
  (interactive)
  (if window-system
      (progn
	(require 'speedbar)
	(speedbar-add-supported-extension ".ads")
	(speedbar-add-supported-extension ".adb")
	;; Show the project files in speedbar (old and new ones)
	(speedbar-add-supported-extension ".apr")
	(speedbar-add-supported-extension ".adp")
 
	(speedbar-frame-mode t))    (message "Speedbar not supported on ordinary terminals."))
  )


(defun glide-setup-colors ()
  (interactive)
  (list-colors-display)
  (customize-face)
  (display-buffer "*Colors*" t))

(defun glide-vi-mode ()
  (interactive)
  (set 'viper-mode t)
  (require 'viper)
  (add-hook 'find-file-hooks 'viper-change-state-to-vi)

  ;;  Make sure that one can use the RET key in the compilation buffers.
  ;;  Do not activate vi-mode in that case
;   (if (member 'compilation-mode viper-vi-state-mode-list)
;       (setcar (member 'compilation-mode viper-vi-state-mode-list) 'ada-mode))
  
  (add-to-list 'viper-emacs-state-mode-list 'compilation-mode)
  )

;;  Load the GUI support mode if found.
(if (locate-library "glide-gui")
    (progn
      (require 'ada-mode)
      (require 'glide-gui "glide-gui")))

;; load the ada-mode

(glide-ada-mode)



;;  Display a glide presentation

(save-excursion
  (set-buffer (get-buffer-create "*Glide*"))
  (require 'cus-edit)
  (let ((inhibit-read-only t)
	pos
	over
	(keymap (make-sparse-keymap))
	(spaces (make-string (/ (- (frame-width) 42) 2) ? ))
	string
	)
    (erase-buffer)
    (insert ""
	    spaces " _       __     __                        \n"
	    spaces "| |     / /__  / /________  ____ ___  ___ \n"
	    spaces "| | /\| / / _ \\/ / ___/ __ \\/ __ `__ \\/ _ \\\n"
	    spaces "| |/ |/ /  __/ / /__/ /_/ / / / / / /  __/\n"
	    spaces "|__/|__/\\___/_/\\___/\\____/_/ /_/ /_/\\___/ \n"
	    "\n")
    (set 'spaces (make-string (/ (- (frame-width) 15) 2) ? ))
    (insert spaces "  ______\n"
	    spaces " /_  __/___\n"
	    spaces "  / / / __ \\\n"
	    spaces " / / / /_/ /\n"
	    spaces "/_/  \\____/\n"
	    "\n")
    (set 'spaces (make-string (/ (- (frame-width) 25) 2) ? ))
    (insert spaces "          ___     __   \n"
	    spaces "   ____ _/ (_)___/ /__ \n"
	    spaces "  / __ `/ / / __  / _ \\\n"
	    spaces " / /_/ / / / /_/ /  __/\n"
	    spaces " \\__, /_/_/\\____/\\___/ \n"
	    spaces "/____/                 \n"
	    "\n\n")
    (set 'spaces (make-string (/ (- (frame-width) 31) 2) ? ))
    (insert spaces "The GNAT Integrated Environment\n")

    ;;  Insert and center version number
    (set 'pos (point))
    (insert "version ")
    (if (file-exists-p glide-version-file)
	(progn
	  (insert-file-contents glide-version-file)
	  (end-of-line)))
    (set 'spaces (make-string (/ (- (frame-width) (- (point) pos )) 2) ? ))
    (beginning-of-line)
    (insert spaces)
    (end-of-line)

    ;;  Add a link to the tutorial

    (insert "\n\nGlide tutorial:     ")
    (set 'pos (point))
    (insert "file://" (expand-file-name data-directory) "tutor/tutorial.html")
    (set 'over (make-overlay pos (point)))
    (overlay-put over 'mouse-face 'highlight)
    (overlay-put over 'face 'bold)
    (define-key keymap [mouse-1] 'browse-url-at-mouse)
    (overlay-put over 'local-map keymap)
    (insert "\n\n\n")
    )    

  ;;  This buffer can never be deleted
  ;; (make-local-variable 'kill-buffer-query-functions)
  ;; (set 'kill-buffer-query-functions
  ;;      (append '((lambda() nil)) kill-buffer-query-functions))

  (set 'buffer-read-only t)
  )

(if (string= (buffer-name) "*scratch*")
    (switch-to-buffer "*Glide*"))

(provide 'glide)

